#!/usr/bin/env python3
"""
Network Vulnerability Scanner
Scans network for common vulnerabilities and security misconfigurations.
Parameters:
    - target: Target IP or network range
    - mode: Scan mode (quick, thorough, stealth)
    - output: Output file for scan results
Requires: scapy, python-nmap packages and admin/root privileges
"""
from scapy.all import *
import nmap
import argparse
import logging
import json
from datetime import datetime
import threading
import queue
import socket
import ssl

class NetworkVulnerabilityScanner:
    def __init__(self, target, mode='quick', output=None):
        self.target = target
        self.mode = mode
        self.output_file = output
        self.vulnerabilities = []
        self.nm = nmap.PortScanner()
        self.setup_logging()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def scan_ports(self):
        try:
            arguments = '-sS -sV'  # SYN scan with version detection
            if self.mode == 'thorough':
                arguments += ' -A'  # Enable OS detection and script scanning
            elif self.mode == 'stealth':
                arguments = '-sS -T2'  # Stealth SYN scan with slower timing

            self.logger.info(f"Starting port scan on {self.target}")
            self.nm.scan(self.target, arguments=arguments)
            
            for host in self.nm.all_hosts():
                for proto in self.nm[host].all_protocols():
                    ports = self.nm[host][proto].keys()
                    for port in ports:
                        service = self.nm[host][proto][port]
                        if 'version' in service and service['version']:
                            self.check_version_vulnerabilities(service)
        except Exception as e:
            self.logger.error(f"Error during port scan: {str(e)}")

    def check_ssl_vulnerabilities(self, host, port):
        try:
            context = ssl.create_default_context()
            with socket.create_connection((host, port)) as sock:
                with context.wrap_socket(sock, server_hostname=host) as ssock:
                    cert = ssock.getpeercert()
                    if cert:
                        self.check_certificate_vulnerabilities(cert)
        except Exception:
            pass

    def check_certificate_vulnerabilities(self, cert):
        try:
            expiry = datetime.strptime(cert['notAfter'], '%b %d %H:%M:%S %Y %Z')
            if expiry < datetime.now():
                self.add_vulnerability('Expired SSL Certificate', 'HIGH')
        except Exception:
            pass

    def check_version_vulnerabilities(self, service):
        if 'version' in service:
            version = service['version']
            # Add known vulnerable versions here
            vulnerable_versions = {
                'OpenSSL': ['1.0.1', '1.0.2'],
                'Apache': ['2.4.48'],
                'nginx': ['1.18.0'],
            }
            
            for product, versions in vulnerable_versions.items():
                if product.lower() in version.lower():
                    for v in versions:
                        if v in version:
                            self.add_vulnerability(
                                f"Vulnerable {product} version: {version}",
                                'HIGH'
                            )

    def add_vulnerability(self, description, severity):
        self.vulnerabilities.append({
            'description': description,
            'severity': severity,
            'timestamp': datetime.now().isoformat()
        })

    def save_results(self):
        if self.output_file:
            try:
                results = {
                    'timestamp': datetime.now().isoformat(),
                    'target': self.target,
                    'mode': self.mode,
                    'vulnerabilities': self.vulnerabilities,
                    'scan_summary': {
                        'total_vulnerabilities': len(self.vulnerabilities),
                        'high_severity': len([v for v in self.vulnerabilities 
                                           if v['severity'] == 'HIGH']),
                        'medium_severity': len([v for v in self.vulnerabilities 
                                             if v['severity'] == 'MEDIUM']),
                        'low_severity': len([v for v in self.vulnerabilities 
                                          if v['severity'] == 'LOW'])
                    }
                }
                
                with open(self.output_file, 'w') as f:
                    json.dump(results, f, indent=4)
                self.logger.info(f"Scan results saved to {self.output_file}")
            except Exception as e:
                self.logger.error(f"Error saving results: {str(e)}")

    def start_scan(self):
        self.logger.info(f"Starting vulnerability scan on {self.target}")
        self.scan_ports()
        self.save_results()
        self.logger.info("Vulnerability scan completed")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Network Vulnerability Scanner")
    parser.add_argument("target", help="Target IP or network range")
    parser.add_argument("--mode", choices=['quick', 'thorough', 'stealth'],
                      default='quick', help="Scan mode")
    parser.add_argument("--output", help="Output file for scan results")
    args = parser.parse_args()

    scanner = NetworkVulnerabilityScanner(args.target, args.mode, args.output)
    scanner.start_scan()
